//---------------------------------------------------------------------------

CL_SceneLayer::CL_SceneLayer(CL_PixelBuffer Buffer, std::string Name) : CL_Surface(Buffer)
{
	m_Position = CL_Vector2(0,0);
	m_Canvas = new CL_Canvas();
	m_Canvas->select_surface(*this);
	m_bVisible = true;
	m_Data = NULL;

	setName(Name);
}

//---------------------------------------------------------------------------

CL_SceneLayer::~CL_SceneLayer()
{
	clearObjects();
}

//---------------------------------------------------------------------------
// void clearLayers()
// Removes all layer objects from the scene manager
// This deletes all layers and their child sprites

void CL_SceneLayer::clearObjects()
{
	for(unsigned int i = 0; i < m_Objects.size(); i++)
	{
		if(m_Objects[i])
			delete m_Objects[i];
	}

	m_Objects.clear();
}

//---------------------------------------------------------------------------
//void composite(CL_Rect *Rect)
//Renders all sprites onto the layer surface
//Called each draw

void CL_SceneLayer::composite(CL_Rect *Rect)
{
	m_Canvas->get_gc()->clear(CL_Color(0, 0, 0, 0));

	//Cycle through all objects to draw
	for(unsigned int i = 0; i < m_Objects.size(); i++)
	{
		CL_SceneObject *Object = m_Objects[i];

		if(Object->getVisible()) //If object visible
		{
			//Test for cull
			CL_Rect LayerRect(0, 0, get_width(), get_height()); //Relative layer dimensions

			CL_Rect CL_SpriteRect((int) Object->getX(), (int) Object->getY(), //Relative object dimensions
								  (int) (Object->getX() + Object->get_width()),
								  (int) (Object->getY() + Object->get_height()));

			if(CL_SpriteRect.is_overlapped(LayerRect)) //If sprite inside layer
			{
				//Draw object

				Object->draw(Rect->left + Object->getX(),
							 Rect->top + Object->getY(),
							 m_Canvas->get_gc());
			}
		}
	}

	m_Canvas->sync_surface();
}

//---------------------------------------------------------------------------

void CL_SceneLayer::draw(float x, float y, CL_GraphicContext* context)
{
	//Layer Rect Properties; absolute top, left, width and height on screen that will be drawn
	CL_Rect Rect;
	Rect.left = (int)(x + m_Position.x);
	Rect.top =  (int)(y + m_Position.y);
	Rect.right = Rect.left + get_width();
	Rect.bottom = Rect.top + get_height();

	composite(&Rect);

	CL_Surface::draw(x, y, context);
}
