#ifndef header_scenemanager
#define header_scenemanager

/*
Scene Manager Class.
*/
//---------------------------------------------------------------------------
template <typename T>
void FromString(T &aValue, const std::string &aStr)
{
   std::stringstream ss(aStr);
   ss >> aValue;
}

//---------------------------------------------------------------------------

template <typename T>
std::string ToString(T aValue)
{
   std::stringstream ss;
   ss << aValue;
   return ss.str();
}

//---------------------------------------------------------------------------

class CL_SceneManager : public CL_Surface
{
private:
		
	std::vector<CL_SceneLayer*> m_Layers;
	CL_Canvas *m_Canvas;
	std::string m_SceneName;
	CL_ResourceManager *m_ResourceManager;

	void composite(CL_Rect *Rect);

protected:
public:

	//Constructors and Destructor
	CL_SceneManager(CL_PixelBuffer Buffer, CL_ResourceManager* Manager = NULL);
	~CL_SceneManager();

	//Layers
	void addLayer(CL_SceneLayer* Layer) {m_Layers.push_back(Layer);}
	CL_SceneLayer* createLayer(float Width = 800, float height=600,
							   CL_PixelFormat Format = CL_PixelFormat::rgba8888, float Pitch = 3200,
							   std::string Name = "");
	void clearLayers();
	CL_SceneLayer* findLayer(std::string Name);
	void swapLayers(unsigned int Index1, unsigned int Index2);
	void swapLayers(std::string Layer1Name, std::string Layer2Name);
	CL_SceneLayer* removeLayer(unsigned int Index);
	CL_SceneLayer* removeLayer(std::string Name);

	//Objects
	CL_SceneObject* createSceneObject(std::string Name, CL_ResourceManager* ResourceManager,
								      float x = 0, float y = 0, bool visible = false);
	CL_SceneObject* findObject(std::string Name);
	void swapObject(CL_SceneLayer* Layer, unsigned int Index1, unsigned int Index2);
	void swapObject(CL_SceneLayer* Layer, std::string Object1Name, std::string Object2Name);
	void swapObject(std::string LayerName, unsigned int Index1, unsigned int Index2);
	void swapObject(std::string LayerName, std::string Object1Name, std::string Object2Name);

	//Draw
	void draw(float x = 0, float y = 0, CL_GraphicContext* context = 0);
	void draw(const CL_Rect& dest, CL_GraphicContext* context = 0);
	void draw(const CL_Rect& src, const CL_Rect& dest, CL_GraphicContext* context = 0);
	void draw(const CL_Surface_DrawParams1& params1, CL_GraphicContext* context = 0);
	void draw(const CL_Surface_DrawParams2& params2, CL_GraphicContext* context = 0);

	//XML
	void saveToXMLFile(std::string FileName);
	void loadFromXMLFile(std::string FileName);
};

#endif